//!	EOS_TxStream.h


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_TxStream_h_
#define _EOS_TxStream_h_


//--------------------------- include section -----------------------
#include "EOS_TxStub.h"
#include <stdarg.h>


//--------------------------- class definition ----------------------
//! CEOS_TxStream represents the sending stub of BYTEs.
//! In addition it provides the C-like function printf
class CEOS_TxStream :
        public CEOS_TxStub<BYTE>
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_TxStream();

    //! \internal
    NO_COPY_AND_ASSIGNMENT(CEOS_TxStream);


//--------------------------- public types --------------------------
public:
    typedef CEOS_TxStream& (*TManipulatorFunction)(CEOS_TxStream&);


//--------------------------- public access -------------------------
public:
    // printf -------------------------------------------------------
    //! similar usage like the C-known function printf
    //! \code
    //! CEOS_TxStream TxStream;
    //! // will generate "Temperature 024°C"
    //! TxStream.printf("Temperature %03d°C\n",24);
    //! \endcode
    void printf(const char* strFormat,
                ...);

    // vprintf ------------------------------------------------------
    //! \internal
    //! parses the format string and calls the formatting functions
    void vprintf(const char* strFormat,
                 va_list& args);




    // oparator << --------------------------------------------------
    inline CEOS_TxStream& operator <<(TManipulatorFunction pFunction) { return (*pFunction)(*this); }

    CEOS_TxStream& operator<<(char ch);
    CEOS_TxStream& operator<<(bool bValue);
    CEOS_TxStream& operator<<(int nValue);
    CEOS_TxStream& operator<<(double fValue);
    CEOS_TxStream& operator<<(const char* strText);


    // manipulators -------------------------------------------------
    friend CEOS_TxStream& endl(CEOS_TxStream&);
    friend CEOS_TxStream& hex(CEOS_TxStream&);
    friend CEOS_TxStream& dec(CEOS_TxStream&);
    friend CEOS_TxStream& oct(CEOS_TxStream&);
    friend CEOS_TxStream& bin(CEOS_TxStream&);
    friend CEOS_TxStream& left(CEOS_TxStream&);
    friend CEOS_TxStream& right(CEOS_TxStream&);

    CEOS_TxStream& width(uint8 nWidth);
    CEOS_TxStream& precision(uint8 nPrecision);

protected:
    CEOS_TxStream& reset();


//--------------------------- protected types -----------------------
protected:
    // UFormat ------------------------------------------------------
    //! \internal
    //! different format types
    typedef union
    {
        BYTE byte;
        struct
        {
            bool bMinusSign     : 1;
            bool bLeftAlign     : 1;
            bool bZeroFill      : 1;
            bool bUppercase     : 1;
        } sFormat;
    } UFormat;


//--------------------------- protected access ----------------------
protected:
    // print --------------------------------------------------------
    //! \internal
    void print(const char* pText);

    // length -------------------------------------------------------
    //! \internal
    size_t length(const char* pText) const;

    // print --------------------------------------------------------
    //! \internal
    void print(unsigned int nValue,
               uint8 nBase,
               char chLetterBase);

    // length -------------------------------------------------------
    //! \internal
    size_t length(unsigned int nValue,
                  uint8 nBase) const;

    // print --------------------------------------------------------
    //! \internal
    //! prints a string to the output buffer
    void print(const char* strText,
               size_t nWidth,
               UFormat uFormat);

    // print --------------------------------------------------------
    //! \internal
    //! prints an integer value to the output
    void print(unsigned int nValue,
               uint8 nBase,
               char chLetterBase,
               size_t nWidth,
               UFormat uFormat);


//--------------------------- protected data ------------------------
protected:
    // m_nBase ------------------------------------------------------
    uint8 m_nBase;

    // m_nWidth -----------------------------------------------------
    uint8 m_nWidth;

    // m_nPrecision -------------------------------------------------
    uint8 m_nPrecision;

    // m_chLetterBase -----------------------------------------------
    char m_chLetterBase;

    // m_uFormat ----------------------------------------------------
    UFormat m_uFormat;
};


//--------------------------- manipulator functions -----------------
extern CEOS_TxStream& endl(CEOS_TxStream&);
extern CEOS_TxStream& hex(CEOS_TxStream&);
extern CEOS_TxStream& dec(CEOS_TxStream&);
extern CEOS_TxStream& oct(CEOS_TxStream&);
extern CEOS_TxStream& bin(CEOS_TxStream&);
extern CEOS_TxStream& left(CEOS_TxStream&);
extern CEOS_TxStream& right(CEOS_TxStream&);


#endif
